@@ -17,9 +17,11 @@
 
 from __future__ import division, print_function, absolute_import
 
+from distutils.version import LooseVersion
+
 import pytest
 
-from hypothesis import core, settings
+from hypothesis import Verbosity, core, settings
 from hypothesis.reporting import default as default_reporter
 from hypothesis.reporting import with_reporter
 from hypothesis.statistics import collector
@@ -27,6 +29,7 @@ from hypothesis.internal.compat import OrderedDict, text_type
 from hypothesis.internal.detection import is_hypothesis_test
 
 LOAD_PROFILE_OPTION = '--hypothesis-profile'
+VERBOSITY_OPTION = '--hypothesis-verbosity'
 PRINT_STATISTICS_OPTION = '--hypothesis-show-statistics'
 SEED_OPTION = '--hypothesis-seed'
 
@@ -52,6 +55,12 @@ def pytest_addoption(parser):
         action='store',
         help='Load in a registered hypothesis.settings profile'
     )
+    group.addoption(
+        VERBOSITY_OPTION,
+        action='store',
+        choices=[opt.name for opt in Verbosity],
+        help='Override profile with verbosity setting specified'
+    )
     group.addoption(
         PRINT_STATISTICS_OPTION,
         action='store_true',
@@ -68,7 +77,7 @@ def pytest_addoption(parser):
 def pytest_report_header(config):
     profile = config.getoption(LOAD_PROFILE_OPTION)
     if not profile:
-        profile = 'default'
+        profile = settings._current_profile
     settings_str = settings.get_profile(profile).show_changed()
     if settings_str != '':
         settings_str = ' -> %s' % (settings_str)
@@ -80,6 +89,16 @@ def pytest_configure(config):
     profile = config.getoption(LOAD_PROFILE_OPTION)
     if profile:
         settings.load_profile(profile)
+    verbosity_name = config.getoption(VERBOSITY_OPTION)
+    if verbosity_name:
+        verbosity_value = Verbosity[verbosity_name]
+        profile_name = '%s-with-%s-verbosity' % (
+            settings._current_profile, verbosity_name
+        )
+        # register_profile creates a new profile, exactly like the current one,
+        # with the extra values given (in this case 'verbosity')
+        settings.register_profile(profile_name, verbosity=verbosity_value)
+        settings.load_profile(profile_name)
     seed = config.getoption(SEED_OPTION)
     if seed is not None:
         try:
@@ -103,7 +122,9 @@ def pytest_runtest_call(item):
         store = StoringReporter(item.config)
 
         def note_statistics(stats):
-            gathered_statistics[item.nodeid] = stats
+            lines = [item.nodeid + ':', ''] + stats.get_description() + ['']
+            gathered_statistics[item.nodeid] = lines
+            item.hypothesis_statistics = lines
 
         with collector.with_value(note_statistics):
             with with_reporter(store):
@@ -120,42 +141,42 @@ def pytest_runtest_makereport(item, call):
             'Hypothesis',
             '\n'.join(item.hypothesis_report_information)
         ))
+    if hasattr(item, 'hypothesis_statistics') and report.when == 'teardown':
+        # Running on pytest < 3.5 where user_properties doesn't exist, fall
+        # back on the global gathered_statistics (which breaks under xdist)
+        if hasattr(report, 'user_properties'):  # pragma: no branch
+            val = ('hypothesis-stats', item.hypothesis_statistics)
+            # Workaround for https://github.com/pytest-dev/pytest/issues/4034
+            if isinstance(report.user_properties, tuple):
+                report.user_properties += (val,)
+            else:
+                report.user_properties.append(val)
 
 
 def pytest_terminal_summary(terminalreporter):
     if not terminalreporter.config.getoption(PRINT_STATISTICS_OPTION):
         return
     terminalreporter.section('Hypothesis Statistics')
-    for name, statistics in gathered_statistics.items():
-        terminalreporter.write_line(name + ':')
-        terminalreporter.write_line('')
 
-        if not statistics.has_runs:
-            terminalreporter.write_line('  - Test was never run')
-            continue
+    if LooseVersion(pytest.__version__) < '3.5':  # pragma: no cover
+        if not gathered_statistics:
+            terminalreporter.write_line(
+                'Reporting Hypothesis statistics with pytest-xdist enabled '
+                'requires pytest >= 3.5'
+            )
+        for lines in gathered_statistics.values():
+            for li in lines:
+                terminalreporter.write_line(li)
+        return
 
-        terminalreporter.write_line((
-            '  - %d passing examples, %d failing examples,'
-            ' %d invalid examples') % (
-            statistics.passing_examples, statistics.failing_examples,
-            statistics.invalid_examples,
-        ))
-        terminalreporter.write_line(
-            '  - Typical runtimes: %s' % (statistics.runtimes,)
-        )
-        terminalreporter.write_line(
-            '  - Fraction of time spent in data generation: %s' % (
-                statistics.draw_time_percentage,))
-        terminalreporter.write_line(
-            '  - Stopped because %s' % (statistics.exit_reason,)
-        )
-        if statistics.events:
-            terminalreporter.write_line('  - Events:')
-            for event in statistics.events:
-                terminalreporter.write_line(
-                    '    * %s' % (event,)
-                )
-        terminalreporter.write_line('')
+    # terminalreporter.stats is a dict, where the empty string appears to
+    # always be the key for a list of _pytest.reports.TestReport objects
+    # (where we stored the statistics data in pytest_runtest_makereport above)
+    for test_report in terminalreporter.stats.get('', []):
+        for name, lines in test_report.user_properties:
+            if name == 'hypothesis-stats' and test_report.when == 'teardown':
+                for li in lines:
+                    terminalreporter.write_line(li)
 
 
 def pytest_collection_modifyitems(items):
